<?php

/**
 * @file
 * Plugin who deals with paths.
 */

$plugin = array(
  'name' => 'Menu Hooks',
  'description' => 'Displays all defined hook_menu() paths.',
  'render callbacks' => array(
    '_moduleinfo_hook_menu_defined' => t('Defined Paths'),
    '_moduleinfo_hook_menu_access' => t('Path Access by Role'),
  ),
);

/**
 * Renders informations to exposed paths.
 *
 * @param string $module_name
 *   Name of the module to display.
 *
 * @return string
 *   Rendered informations.
 */
function _moduleinfo_hook_menu_defined($module_name = NULL) {
  $list = module_implements('menu');

  if (!is_null($module_name) && !in_array($module_name, $list)) {
    return '';
  }

  if (!is_null($module_name)) {
    $list = array($module_name);
  }

  $items = array();
  foreach ($list as $module_name) {
    $hook_menu = $module_name . '_menu';
    if (!function_exists($hook_menu)) {
      continue;
    }

    $paths = $hook_menu();

    foreach ($paths as $path => $def) {
      $tmp = array();
      if (strpos($path, '%') !== FALSE) {
        $tmp['info'] = '<span class="mono">' . $path . '</span>';
      }
      else {
        $tmp['info'] = '<span class="mono">' . l($path, $path) . '</span>';
      }

      $title = '';
      if (isset($def['title'])) {
        $title = '<b>' . $def['title'] . '</b>';
      }
      $description = '';
      if (isset($def['description'])) {
        $description = $def['description'];
      }
      if (isset($def['page callback']) && $def['page callback'] == 'drupal_get_form') {
        $tmp['info'] .= ' (' . t('Form') . ')';
      }

      if (!empty($title) || !empty($description)) {
        $tmp['desc'] = $title . ' ' . $description;
      }

      $items[$path] = $tmp;
    }
  }

  ksort($items);

  $description = t('Lists all paths defined via hook_menu() of that module.');
  if (is_null($module_name)) {
    $description = t('Lists all paths defined via hook_menu().');
  }

  return _moduleinfo_helper_fieldset($items, array(
    '#title' => t('hook_menu'),
    '#description' => $description,
  ));
}

/**
 * Renders infomrations of which role can access a path.
 *
 * @param string $module_name
 *   Name of the module to display.
 *
 * @return string
 *   Rendered informations.
 */
function _moduleinfo_hook_menu_access($module_name = NULL) {
  $list = module_implements('menu');

  if (!is_null($module_name) && !in_array($module_name, $list)) {
    return '';
  }

  if (!is_null($module_name)) {
    $list = array($module_name);
  }

  $roles = user_roles();
  $permissions = user_role_permissions($roles);

  $items = array();
  foreach ($roles as $role_id => $role) {
    foreach ($list as $module_name) {
      $hook_menu = $module_name . '_menu';
      if (!function_exists($hook_menu)) {
        continue;
      }

      $paths = $hook_menu();

      foreach ($paths as $path => $def) {
        if (isset($def['access callback']) && $def['access callback'] != 'user_access') {
          continue;
        }
        else {
          if (!isset($def['access arguments'])) {
            $def['access arguments'] = 'access content';
          }

          $menu_permission = $def['access arguments'][0];
          if (!in_array($menu_permission, array_keys($permissions[$role_id]))) {
            continue;
          }
        }

        $tmp = array();
        if (strpos($path, '%') !== FALSE) {
          $tmp['info'] = '<span class="mono">' . $path . '</span>';
        }
        else {
          $tmp['info'] = '<span class="mono">' . l($path, $path) . '</span>';
        }

        $title = '';
        if (isset($def['title'])) {
          $title = '<b>' . $def['title'] . '</b>';
        }
        $description = '';
        if (isset($def['description'])) {
          $description = $def['description'];
        }
        if (isset($def['page callback']) && $def['page callback'] == 'drupal_get_form') {
          $tmp['info'] .= ' (' . t('Form') . ')';
        }

        if (!empty($title) || !empty($description)) {
          $tmp['desc'] = $title . ' ' . $description;
        }

        $items[$role][$path] = $tmp;
      }
    }
  }

  ksort($items);

  $fieldsets = array();
  $pathlist = array();
  foreach ($items as $role => $paths) {
    foreach ($paths as $path => $info) {
      $pathlist[$role][] = array(
        'info' => $info['info'],
      );
    }

    $fieldsets[$role] = _moduleinfo_helper_fieldset($pathlist[$role], array(
      '#title' => $role,
      '#description' => t('These paths are accessible by the role !role', array(
        '!role' => '<span class="mono">' . $role . '</span>',
      )),
    ));
  }

  $description = t('I would like to provide a list of paths who are accessible via role, but there is no standard and some custom access functions not even allow to provide a user object. Therefore only a list of paths is provided by role with a direct role permissions array test.');

  return _moduleinfo_helper_fieldset(implode('', $fieldsets), array(
    '#title' => t('Menu Access by Role'),
    '#description' => $description,
  ));
}
