<?php

/**
 * @file
 * Plugin who deals with hooks in general.
 */

$plugin = array(
  'name' => 'API Hooks',
  'description' => 'Scans Drupal for defined HOOKs',
  'scan callbacks' => array(
    '_moduleinfo_hooks_scan' => array(
      'scan' => array(
        '/.+\.api\.php$/i',
      ),
      'skip' => array(),
    ),
  ),
  'render callbacks' => array(
    '_moduleinfo_hooks_exposed' => t('Exposed Hooks'),
    '_moduleinfo_hooks_implemented' => t('Implemented Hooks'),
  ),
  'post scan callbacks' => array(
    '_moduleinfo_hooks_post_scan_hook_list',
  ),
  '#weight' => -10,
);

/**
 * Scans api.php files for informations to exposed hooks.
 *
 * Creates a cache with scan_key: 'api hooks'
 *
 * @param string $module_name
 *   Name of the current module
 * @param string $content
 *   Content of the current processed file
 * @param string $uri
 *   URI of the file.
 */
function _moduleinfo_hooks_scan($module_name, $content, $uri) {
  $ref = new PhpReflector($content);
  $ref->parse();
  $symbols = $ref->getResult();

  if (!isset($symbols['defined functions'])) {
    return;
  }

  $functions = $symbols['defined functions'];

  foreach ($functions as $i => $function) {
    // We ignore everything what does not start with hook_
    if (strpos($function->name, 'hook_') !== 0) {
      continue;
    }

    $function_name = $function->name;

    $record[$function_name] = array(
      'shortinfo' => $function->doc_short,
      'doc' => $function->doc,
      'name' => $function_name,
      'signature' => $function->signature,
      'line_number' => $function->line_number,
      'type' => strpos($uri, 'modules/') === 0 ? 'core' : 'user',
      'file' => basename($uri),
      'uri' => $uri,
    );
  }

  if (count($record)) {
    _moduleinfo_cache_write('api hooks', $record, $module_name);
  }
}

/**
 * Renders all exposed hooks by module.
 *
 * @param string $module_name
 *   Name of the module to display.
 *
 * @return string
 *   Rendered informations.
 */
function _moduleinfo_hooks_exposed($module_name = NULL) {
  $result = _moduleinfo_cache_read('api hooks', $module_name);

  if (!count($result)) {
    return '';
  }

  $items = array();
  foreach ($result as $module) {
    foreach ($module->data as $hook_name => $info) {
      $items[$hook_name] = array(
        'info' => '<b class="mono">' . l($info['signature'], 'admin/reports/moduleinfo/info/' . $hook_name . '/hook', array('query' => array('destination' => $_GET['q']))) . '</b>',
        'desc' => $info['shortinfo'],
      );
    }
  }

  ksort($items);

  $description = t('Lists all available hooks of that module based on *.api.php files.');
  if (is_null($module_name)) {
    $description = t('Lists all available hooks in the system by all enabled module based on *.api.php files.');
  }

  return _moduleinfo_helper_fieldset($items, array(
    '#title' => t('Exposed Hooks'),
    '#description' => $description,
  ));
}

/**
 * Renders all implemented hooks by module.
 *
 * @param string $module_name
 *   Name of the module to display.
 *
 * @return string
 *   Rendered informations.
 */
function _moduleinfo_hooks_implemented($module_name = NULL) {
  if (is_null($module_name)) {
    return '';
  }

  $hooks = _moduleinfo_cache_read('all hooks');
  $funcs = _moduleinfo_cache_read('def functions', $module_name);

  if (!count($hooks)) {
    return '';
  }
  if (!count($funcs)) {
    return '';
  }

  $hook_list = array_keys($hooks['0']->data);
  sort($hook_list);
  $hooks = $hooks['0']->data;
  $func_list = $funcs[$module_name]->data;

  $items = array();
  foreach ($func_list as $function_name => $def) {
    if (strpos($function_name, $module_name) !== 0) {
      continue;
    }
    $hook_name = preg_replace("/^{$module_name}_/i", 'hook_', $function_name);

    // FIXME: All hooks with _DELTA_ or similar stuff in there are skipped at
    // the moment try to come up with a way to get these too.
    if (!in_array($hook_name, $hook_list)) {
      continue;
    }

    $items[] = array(
      'info' => '<b class="mono">' . l($def['signature'], 'admin/reports/moduleinfo/info/' . $function_name . '/function', array('query' => array('destination' => $_GET['q']))) . '</b>',
      'desc' => t('Exposed in !module / hook is !hook', array(
        '!module' => '<b class="mono">' . $hooks[$hook_name] . '</b>',
        '!hook' => '<b class="mono">' . $hook_name .  '</b>',
      )),
    );
  }

  return _moduleinfo_helper_fieldset($items, array(
    '#title' => t('Implemented Hooks'),
    '#description' => t('Lists all the implemented hooks by that module based on all exposed hooks in the system.'),
  ));
}

/**
 * Creates a cache for all hooks in one place.
 */
function _moduleinfo_hooks_post_scan_hook_list() {
  $result = _moduleinfo_cache_read('api hooks');

  if (!count($result)) {
    return;
  }

  $hooks = array();
  foreach ($result as $module_name => $cache) {
    foreach ($cache->data as $hook_name => $info) {
      $hooks[$hook_name] = $module_name;
    }
  }

  _moduleinfo_cache_write('all hooks', $hooks, NULL);
}
